package com.jiuqi.util;

package .tmp;

import java.util.Arrays;

public final class IntList {
  protected int size;
  
  protected int[] data;
  
  public IntList() {}
  
  public IntList(int capacity) {
    setCapacity(capacity);
  }
  
  public int size() {
    return this.size;
  }
  
  public void reset() {
    this.size = 0;
  }
  
  public void clear() {
    this.size = 0;
    this.data = null;
  }
  
  public int get(int index) {
    return this.data[index];
  }
  
  public void set(int index, int value) {
    this.data[index] = value;
  }
  
  public int add(int value) {
    int result = this.size;
    setSize(this.size + 1);
    this.data[result] = value;
    return result;
  }
  
  public void insert(int index, int value) {
    setSize(this.size + 1);
    for (int i = this.size - 1; i > index; i--)
      this.data[i] = this.data[i - 1]; 
    this.data[index] = value;
  }
  
  public void remove(int index) {
    for (int i = index; i < this.size - 1; i++)
      this.data[i] = this.data[i + 1]; 
    setSize(this.size - 1);
  }
  
  public void setSize(int value) {
    if (value <= 0) {
      removeAll();
    } else if (value != this.size) {
      if (this.data == null || this.data.length < value) {
        this.data = resize(this.data, value);
      } else if (value > this.size) {
        for (int i = this.size; i < value; i++)
          this.data[i] = 0; 
      } 
      this.size = value;
    } 
  }
  
  public void setCapacity(int value) {
    if (value >= this.size)
      this.data = resize(this.data, value); 
  }
  
  public void removeAll() {
    this.size = 0;
  }
  
  public void setAll(int value) {
    if (this.data != null)
      Arrays.fill(this.data, value); 
  }
  
  public void plus(int index, int value) {
    this.data[index] = this.data[index] + value;
  }
  
  public void setSmaller(int index, int value) {
    if (value < this.data[index])
      this.data[index] = value; 
  }
  
  public void setLarger(int index, int value) {
    if (value > this.data[index])
      this.data[index] = value; 
  }
  
  public int getTotal() {
    int value = 0;
    for (int i = 0; i < this.size; i++)
      value += this.data[i]; 
    return value;
  }
  
  public int getTotal(int fromIndex, int toIndex) {
    int value = 0;
    for (int i = fromIndex; i <= toIndex; i++)
      value += this.data[i]; 
    return value;
  }
  
  public boolean isEmpty() {
    return (this.size == 0);
  }
  
  public void assign(com.jiuqi.util.IntList source) {
    setSize(source.size);
    copy(source.data, this.data, this.size);
  }
  
  public int indexOf(int value) {
    for (int i = 0; i < this.size; i++) {
      if (this.data[i] == value)
        return i; 
    } 
    return -1;
  }
  
  public void removeValue(int value) {
    int index = indexOf(value);
    if (index >= 0)
      remove(index); 
  }
  
  public int find(int value) {
    int sb = 0;
    int se = this.size - 1;
    int flag = -1;
    while (sb <= se) {
      int sp = sb + se >> 1;
      flag = value - this.data[sp];
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        continue;
      } 
      sb = sp;
      break;
    } 
    return (flag == 0) ? sb : (-sb - 1);
  }
  
  public int findFirst(int value) {
    int sb = 0;
    int se = this.size - 1;
    int flag = -1;
    while (sb <= se) {
      int sp = sb + se >> 1;
      flag = value - this.data[sp];
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        continue;
      } 
      se = sp;
      if (sb == sp)
        break; 
    } 
    return (flag == 0) ? sb : (-sb - 1);
  }
  
  public int findLast(int value) {
    int sb = 0;
    int se = this.size - 1;
    int flag = -1;
    while (sb <= se) {
      int sp = sb + se + 1 >> 1;
      flag = value - this.data[sp];
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        continue;
      } 
      sb = sp;
      if (se == sp)
        break; 
    } 
    return (flag == 0) ? sb : (-sb - 1);
  }
  
  public void sortAdd(int value) {
    int index = find(value);
    if (index < 0)
      insert(-index - 1, value); 
  }
  
  public void sortAddLast(int value) {
    int index = findLast(value);
    if (index < 0) {
      insert(-index - 1, value);
    } else {
      insert(index + 1, value);
    } 
  }
  
  public void exchange(int index1, int index2) {
    if (index1 != index2) {
      int t = this.data[index1];
      this.data[index1] = this.data[index2];
      this.data[index2] = t;
    } 
  }
  
  public void move(int index1, int index2) {
    if (index1 > index2) {
      int t = this.data[index1];
      for (int i = index1; i > index2; i--)
        this.data[i] = this.data[i - 1]; 
      this.data[index2] = t;
    } else if (index1 < index2) {
      int t = this.data[index1];
      for (int i = index1; i < index2; i++)
        this.data[i] = this.data[i + 1]; 
      this.data[index2] = t;
    } 
  }
  
  public int getFirst() {
    return this.data[0];
  }
  
  public int getLast() {
    return this.data[this.size - 1];
  }
  
  public void push(int value) {
    add(value);
  }
  
  public int pop() {
    return this.data[this.size--];
  }
  
  public int[] toArray() {
    if (this.size == 0)
      return null; 
    int[] result = new int[this.size];
    copy(this.data, result);
    return result;
  }
  
  public int toArray(int[] data) {
    copy(this.data, data, this.size);
    return this.size;
  }
  
  public static int[] resize(int[] data, int newSize) {
    if (newSize < 256) {
      newSize = newSize + 63 & 0xFFFFFFC0;
    } else if (newSize < 4096) {
      newSize = newSize + 255 & 0xFFFFFF00;
    } else {
      newSize = newSize + 4095 & 0xFFFFF000;
    } 
    int[] newData = new int[newSize];
    if (data != null)
      copy(data, newData); 
    return newData;
  }
  
  public static void copy(int[] src, int[] dst) {
    if (src == null || dst == null)
      return; 
    int count = src.length;
    if (count > dst.length)
      count = dst.length; 
    for (int i = 0; i < count; i++)
      dst[i] = src[i]; 
  }
  
  public static void copy(int[] src, int[] dst, int count) {
    if (src == null || dst == null)
      return; 
    if (count > src.length)
      count = src.length; 
    if (count > dst.length)
      count = dst.length; 
    for (int i = 0; i < count; i++)
      dst[i] = src[i]; 
  }
  
  public String toString() {
    StringBuffer sb = new StringBuffer(size() * 4);
    sb.append('(');
    for (int i = 0; i < this.size; i++) {
      sb.append(this.data[i]);
      if (i < this.size - 1)
        sb.append(','); 
    } 
    sb.append(")");
    return sb.toString();
  }
}
